#!/bin/sh



# User is root?


if [ "$UID" -ne "0" ]
then
echo
echo
echo "You must be root to run this script"
echo
echo
exit
fi

# Asking for OS type

clear
echo
echo "		##########################################"
echo
echo "                       Welcome to GRASSLinks3.2beta"
echo
echo "		##########################################"
echo
echo "                This is the GRASSLinks installation script"
echo
echo
echo
echo
echo "Please enter 1 or 2 to choose OS type"
echo
echo "1 - Debian-based or other system"
echo "2 - Older (Apache 1.x) Red Hat-based system"
echo
echo
echo
echo 
echo
echo
echo
echo "Enter your OS type (REQUIRED):"
read OS_TYPE


# Checking user input. Note string comparation on second []. Note 
# use of || (logical OR) out of brackets and use of -a (logical AND)
# into the brackets

if [ ! "$OS_TYPE" ] || [ "$OS_TYPE" != "1" -a "$OS_TYPE" != "2" ] 
then
echo
echo
echo "You must choose 1 or 2 as your operating system type"
echo
echo "			Script aborted"
echo
exit 
fi









# Asking for e-mail address

clear
echo
echo
echo "                        GRASSLinks3.2beta installation"
echo
echo "                               e-mail address"
echo
echo
echo
echo
echo "  Please enter your e-mail address"
echo
echo "  Grasslinks will send messages to this address with useful"
echo " informations"
echo   
echo 
echo 
echo
echo 
echo  
echo  
echo   
echo
echo
echo "Enter e-mail address (REQUIRED):"
read EMAIL



# Checking user input

if [ ! "$EMAIL" ]
then
echo
echo
echo "You must enter your e-mail address"
echo
echo "			Script aborted"
echo
exit 

else 
VALID_EMAIL=`echo $EMAIL | grep [a-z0-9]*@[a-z0-9]*` 

if [ ! "$VALID_EMAIL" ]
then
echo
echo
echo "You must enter a valid e-mail address, in the form"
echo "          user@host.domain"
echo 
echo 
echo "			Script aborted"
echo
echo
exit

fi

fi









# Asking for non-privileged username

clear
echo
echo
echo "                        GRASSLinks3.2beta installation"
echo
echo "                           Non-privileged username"
echo
echo
echo
echo
echo "  Please enter non-privileged username and group (user:group)"
echo
echo "  These should be the same as for owner of GRASS data directory"
echo " (where GRASS locations are) or at least have read/write"
echo " permissions on it"  
echo 
echo 
echo
echo 
echo  
echo  
echo   
echo
echo
echo "Enter non-privileged username:group (REQUIRED):"
read NP_USER



# Checking user input

if [ ! "$NP_USER" ]
then
echo
echo
echo "You must enter a non-privileged user:group"
echo
echo "			Script aborted"
echo
exit 

else 
VALID_USER=`echo $NP_USER | grep [a-zA-Z0-9]*:[a-zA-Z0-9]*` 

if [ ! "$VALID_USER" ]
then
echo
echo
echo "You must give user and group as user:group"
echo
echo "Note the double colon, it's for POSIX compliance"
echo 
echo "			Script aborted"
echo
echo
exit
fi

fi







# Asking for webserver username

clear
echo
echo
echo "                        GRASSLinks3.2beta installation"
echo
echo "                  	   Apache username and group"
echo
echo
echo
echo
echo "  Please enter username and group under apache runs."
echo
echo "  This is typically www-data:www-data on debian-based systems"
echo " and nobody:nobody on Apache 1.x Red Hat-based systems. This value"  
echo " will be set by default now, based on your previous choice"
echo " about OS type"
echo 
echo "  You can safely enter nothing here to accept this default value"
echo  
echo "  If you have different values on your httpd.conf, you should"
echo " enter them now"
echo
echo
echo
echo "Enter webserver username:group (OPTIONAL):"
read WEB_USER



# If user inputs nothing, set default values based on OS_TYPE

if [ ! "$WEB_USER" ]
then

case "$OS_TYPE" in
"1" )
WEB_USER=www-data:www-data
;;
"2" )
WEB_USER=nobody:nobody
;;
esac

else 
VALID_USER=`echo $WEB_USER | grep [a-zA-Z0-9]*:[a-zA-Z0-9]*` 

if [ ! "$VALID_USER" ]
then
echo
echo
echo "You must give user and group as user:group"
echo
echo "Note the double colon, it's for POSIX compliance"
echo 
echo "			Script aborted"
echo
echo
exit
fi

fi


# Asking for htdocs directory 

clear
echo
echo
echo "                        GRASSLinks3.2beta installation"
echo
echo "                  	     htdocs directory"
echo
echo
echo
echo
echo "  Please enter full path to your htdocs directory."
echo
echo "  This is typically /var/www on debian-based systems"
echo " and /home/httpd/html on Apache 1.x Red Hat-based systems. This value"  
echo " will be set by default now, based on your previous choice"
echo " about OS type"
echo 
echo "  You can safely enter nothing here to accept this default value"
echo " if you have installed apache as .deb or .rpm package"
echo
echo 
echo
echo
echo
echo "Enter full path to htdocs directory (OPTIONAL):"
read DOCS_DIR



# If user inputs nothing, set default values based on OS_TYPE

if [ ! "$DOCS_DIR" ]
then

case "$OS_TYPE" in
"1" )
DOCS_DIR=/var/www
;;
"2" )
DOCS_DIR=/home/httpd/html
;;
esac

fi

# Checking if path is absolute 

VALID_PATH=`echo $DOCS_DIR | grep ^/`

if [ ! "$VALID_PATH" ] 
then
echo
echo
echo " $DOCS_DIR is a relative path (no / at start)"
echo
echo " You must give an absolute path"
echo
echo " Script aborted"
echo
exit

elif [ ! -d "$VALID_PATH" ] # Check if $VALID_PATH is really a directory
then
echo
echo
echo "$VALID_PATH: No such directory"
echo
exit
fi










# Asking for cgi-bin directory 

clear
echo
echo
echo "                        GRASSLinks3.2beta installation"
echo
echo "                  	     cgi-bin directory"
echo
echo
echo
echo
echo "  Please enter full path to your cgi-bin directory."
echo
echo "  This is typically /usr/lib/cgi-bin on debian-based systems"
echo " and /home/httpd/cgi-bin on Apache 1.x Red Hat-based systems. This value"  
echo " will be set by default now, based on your previous choice"
echo " about OS type"
echo 
echo "  You can safely enter nothing here to accept this default value"
echo " if you have installed apache as .deb or .rpm package"
echo
echo 
echo
echo
echo
echo "Enter full path to cgi-bin directory (OPTIONAL):"
read CGI_DIR



# If user inputs nothing, set default values based on OS_TYPE

if [ ! "$CGI_DIR" ]
then

case "$OS_TYPE" in
"1" )
CGI_DIR=/usr/lib/cgi-bin
;;
"2" )
CGI_DIR=/home/httpd/cgi-bin
;;
esac

fi



# Checking if path is absolute 

VALID_PATH=`echo $DOCS_DIR | grep ^/`

if [ ! "$VALID_PATH" ] 
then
echo
echo
echo " $DOCS_DIR is a relative path (no / at start)"
echo
echo " You must give an absolute path"
echo
echo " Script aborted"
echo
exit

elif [ ! -d "$VALID_PATH" ] # Check if $VALID_PATH is really a directory
then
echo
echo
echo "$VALID_PATH: No such directory"
echo
exit
fi








# Asking for grass data directory 

clear
echo
echo
echo "                        GRASSLinks3.2beta installation"
echo
echo "                  	     GRASS data directory"
echo
echo
echo
echo
echo "  Please enter full path to grass data directory. This is where"
echo " your GRASS locations are. Example: /usr/local/data "
echo 
echo   
echo
echo 
echo 
echo
echo
echo
echo 
echo
echo
echo
echo "Enter GRASS data directory path (REQUIRED):"
read GRASS_DATA_DIR



# Checking user input

if [ ! "$GRASS_DATA_DIR" ]
then
echo
echo
echo "You must enter path to your GRASS data directory"
echo
echo "			Script aborted"
echo
exit 
fi




# Checking if path is absolute 

VALID_PATH=`echo $DOCS_DIR | grep ^/`

if [ ! "$VALID_PATH" ] 
then
echo
echo
echo " $DOCS_DIR is a relative path (no / at start)"
echo
echo " You must give an absolute path"
echo
echo " Script aborted"
echo
exit

elif [ ! -d "$VALID_PATH" ] # Check if $VALID_PATH is really a directory
then
echo
echo
echo "$VALID_PATH: No such directory"
echo
exit
fi











# Asking for grass binary directory

clear
echo
echo
echo "                        GRASSLinks3.2beta installation"
echo
echo "                  	     GRASS binary directory"
echo
echo
echo
echo
echo "  Please enter full path to grass binary directory. This is where"
echo " your GRASS programs are"
echo
echo "  This directory is typically /usr/local/grass5. This value"
echo " will be set by default, based on your previous choice about"
echo " OS type"
echo
echo "  You can safely enter nothing here to accept this default value,"
echo " unless you gived an specific value to GRASS configure script"
echo " (or installed GRASS as .deb or .rpm package)" 
echo
echo
echo
echo "Enter GRASS binary directory path (OPTIONAL):"
read GRASS_BIN_DIR



# If user inputs nothing, set to default value

if [ ! "$GRASS_BIN_DIR" ]
then
GRASS_BIN_DIR=/usr/local/grass5
fi



# Checking if path is absolute 

VALID_PATH=`echo $DOCS_DIR | grep ^/`

if [ ! "$VALID_PATH" ] 
then
echo
echo
echo " $DOCS_DIR is a relative path (no / at start)"
echo
echo " You must give an absolute path"
echo
echo " Script aborted"
echo
exit

elif [ ! -d "$VALID_PATH" ] # Check if $VALID_PATH is really a directory
then
echo
echo
echo "$VALID_PATH: No such directory"
echo
exit
fi





# Asking for examples (global, spearfish) directory

clear
echo
echo
echo "                        GRASSLinks3.2beta installation"
echo
echo "                  	     examples directory"
echo
echo
echo
echo
echo "  Have you downloaded global and spearfish example datasets?"
echo " This is recommended if you are starting with GRASSLinks"
echo
echo "  You can download global and spearfish from these URLs:"
echo
echo "          http://ippc2.orst.edu/glinks/global.tar.gz"
echo "          http://ippc2.orst.edu/glinks/spearfish.tar.gz"
echo
echo "  These are special versions tuned for use with GRASSLinks. If"
echo " you want to use them (and have downloaded them), please enter full"
echo " path to where they are. Both datasets must be in the same"
echo " directory. Example: /usr/local/data"
echo 
echo "  If you don't like these example datasets, enter nothing here"
echo  
echo
echo
echo
echo "Enter full path to examples directory (OPTIONAL):"
read EX_DIR



# If user inputs nothing, set to default value

if [ ! "$EX_DIR" ]
then
EX_DIR=n

else
VALID_PATH=`echo $EX_DIR | grep ^/`

if [ ! "$VALID_PATH" ] 
then
echo
echo
echo " $EX_DIR is a relative path (no / at start)"
echo
echo " You must give an absolute path"
echo
echo " Script aborted"
echo
exit

elif [ ! -d "$VALID_PATH" ] # Check if $VALID_PATH is really a directory
then
echo
echo
echo "$VALID_PATH: No such directory"
echo
exit
fi

fi






# Asking for user datasets

clear
echo
echo
echo "                        GRASSLinks3.2beta installation"
echo
echo "                  	     user datasets"
echo
echo
echo
echo
echo "  If you have your own GRASS locations, this script can do the"
echo " necessary settings for use them in GRASSLinks"
echo
echo 
echo 
echo
echo
echo
echo
echo 
echo
echo
echo
echo "Do necessary settings? [y | n]:"
read USERDATA



# If user inputs nothing, set to default value

if [ ! "$USERDATA" ]
then
USERDATA=y
fi








# Asking for confirmation on user options

clear
echo
echo
echo "                        GRASSLinks3.2beta installation"
echo
echo "                             These are your options"
echo
echo
echo
echo
echo

if [ "$OS_TYPE" -eq 1 ]
then
echo " System			Debian or other based"
else
echo " System			Apache 1.x Red Hat-based"
fi

echo " Email:			$EMAIL"
echo " Non-privileged user:	$NP_USER"
echo " Apache user:		$WEB_USER"
echo " htdocs directory:	$DOCS_DIR"
echo " cgi-bin directory:	$CGI_DIR"
echo " GRASS data directory:	$GRASS_DATA_DIR"
echo " GRASS bin directory:	$GRASS_BIN_DIR"
echo " Example directory:	$EX_DIR"
echo " User datasets:		$USERDATA"
echo
echo
echo
echo "Are these options correct? [y | n] :"
read CONFIRM


# If user inputs nothing, set CONFIRM to default value (y)

if [ ! "$CONFIRM" ]
then
CONFIRM=y
fi




if [ "$CONFIRM" == "y" ]
then
echo " Installation of GRASSLinks3.2beta has been started..."












# Make shure where we are

cd $DOCS_DIR/glinks


# Setting permissions on glinks directory

chown -R $NP_USER .
chown -R $WEB_USER ./tmp	# web user needs write permission here
chown -R $WEB_USER ./legendGifs		# and here


# Tuning files on glinks directory

cp gl.html index.html
cp gl_matrix.html GL_matrix.html


# Working on glinks/progs directory

cd progs
cp GL_list.default GL_list
cp glinks.pl glconf.pl GL_list GL_forms.pl $CGI_DIR
cp lookitv.sh $GRASS_BIN_DIR/scripts/
cp ppm $GRASS_BIN_DIR/etc/paint/driver.sh/



# Checking owner on $GRASS_BIN_DIR/script and setting lookitv.sh
# the same; idem for ppm file

OWN=`ls -l $GRASS_BIN_DIR/scripts/demo.sh | awk '{print $3 ":" $4}'`
chown $OWN $GRASS_BIN_DIR/scripts/lookitv.sh
chown $OWN $GRASS_BIN_DIR/etc/paint/driver.sh/ppm



# Tuning files moved to $CGI_DIR. Note sed substitutions use " to
# allow use of variables. Note too use of | in place of / for
# easy path substitutions

cd $CGI_DIR
cat glconf.pl | sed \
		-e "s|/home/httpd/html|$DOCS_DIR|" \
		-e "s/coopl\\\@bcc.orst.edu/$EMAIL/" \
		-e "s|/home/httpd/cgi-bin|$CGI_DIR|" \
		-e "s|/home2/data|$GRASS_DATA_DIR|" >> glconf2.pl


# VERY UGLY thing to get the @ escaped (needed for Perl?) only
# on lines with "mailto" If you know how to do this better,
# please let me know!

cat glconf2.pl | sed -e '/mailto/s/@/\\\@/' >> glconf3.pl


mv glconf.pl glconf.pl.bak
rm glconf2.pl
mv glconf3.pl glconf.pl


chown $NP_USER glinks.pl glconf.pl GL_forms.pl GL_list
chmod 755 glinks.pl glconf.pl GL_forms.pl
chmod 644 GL_list


# Install global and spearfish unless user previously says "no"

if [ "$EX_DIR" != "n" ]
then

cd "$GRASS_DATA_DIR"  
tar xvzf $EX_DIR/global.tar.gz
tar xvzf $EX_DIR/spearfish.tar.gz

for i in global spearfish
do
cd $i

for a in `ls -d grasslinks*`
do
cd $a
if [ -f .grassrc5 ]
then
cat .grassrc5 | sed -e "s|/home2/data|$GRASS_DATA_DIR|" >> grassrc5
rm .grassrc5
mv grassrc5 .grassrc5
fi			# end grassrc5 test
cd ..
done	# end grasslinks* loop
cd ..
done 	# end global-spearfish loop


chown -R $NP_USER global
chown -R $NP_USER spearfish
chown -R $WEB_USER global/grasslinks*
chown -R $WEB_USER spearfish/grasslinks*

fi






##############################################################

# Working on user's locations

#############################################################


if [ $USERDATA == "y" ]
then



# Array to hold project codes. Note there is not x y since
# they are the codes of global and spearfish

projects_array=( a b c d e f g h i j k l m n o p q r s t u v w z )

COUNT=0





# Begin customized index.html creation
# begin.txt is html with head of page and forms for
# global and spearfis in an unterminated table

mv $DOCS_DIR/glinks/index.html $DOCS_DIR/glinks/index.bak
cp $DOCS_DIR/glinks/begin.txt $DOCS_DIR/glinks/index.html




# backup GL_list too

cp $CGI_DIR/GL_list $CGI_DIR/GL_list.bak




# Now go to locations, check for directories other than global and spearfish
# and make sure there is a PERMANENT directory so whe are on a location 

cd "$GRASS_DATA_DIR"
echo `pwd`

for i in `ls`
do


if [ -d "$i" -a "$i" != "global" -a "$i" != "spearfish" ]  # main condition
then
cd $i
echo `pwd`

if [ -d "PERMANENT" ]	# main condition 2, makes sure this is a location
then



############### Making grasslinks mapsets ##################

echo "Making GRASSLinks mapsets on $i"

for c in 0 1 2 3
do
mkdir grasslinks"$c"
cd grasslinks"$c"
echo "GISDBASE: $GRASS_DATA_DIR" >> .grassrc5
echo "LOCATION_NAME: $i" >> .grassrc5
echo "LOCATION: $GRASS_DATA_DIR/$i/grasslinks$a" >> .grassrc5
echo "PAINTER: ppm" >> .grassrc5
echo "MAPLP: pmap.grasslinks$c" >> .grassrc5
echo "MAPSET: grasslinks$c" >> .grassrc5
touch UNLOCK
chown -R $WEB_USER .
cd ..
done    # End of loop 1 2 3

############# End of mapsets creation ######################








#################### Making GL_list #############################


# Go into PERMANENT, set variables for change on each iteration

cd PERMANENT
echo `pwd`

PROJECT=${projects_array[$COUNT]}
LOC=$i
META_DIR=$DOCS_DIR/glinks/meta/$LOC
THUMB_DIR=$DOCS_DIR/glinks/tnails/$LOC
META=$LOC
#META=/meta/$LOC
THUMB=/tnail/$LOC
mkdir $META_DIR
mkdir $THUMB_DIR




# Use these variables to concatenate customized forms to index.html

cat $DOCS_DIR/glinks/form.txt | sed	-e "s/GGGGG/$PROJECT/" \
					-e "s/HHHHH/$i/" >> $DOCS_DIR/glinks/index.html




# List location as new project on GL_list

echo "project|$PROJECT|$LOC dataset||$LOC|default@PERMANENT" >> $CGI_DIR/GL_list




for a in `ls -d *`	# secondary loop
do

if [ "$a" == cell ]	# secondary condition 1
then
cd $a
echo `pwd`


for b in `ls`			# loop to make new raster lines on GL_list
do
echo "raster|$PROJECT|$b@PERMANENT|$b|$META/$b.txt|$THUMB/$b.png||" >> $CGI_DIR/GL_list
echo "Not yet done. You should edit this file to enter relevant metadata" > "$META_DIR"/"$b".txt
done
cd ..




elif [ "$a" == dig ]	# secondary condition 2
then
cd $a
echo `pwd`

for b in `ls`			# loop to make new vector lines in GL_list
do
echo "vector|$PROJECT|$b@PERMANENT|$b|$META/$b.txt|||" >> $CGI_DIR/GL_list
echo "Not yet done. You should edit this file to enter relevant metadata" > "$META_DIR"/"$b".txt
done
cd ..




elif [ "$a" == site_list ]	# secondary condition 3
then
cd $a
echo `pwd`

for b in `ls`		# loop to make new site lines in GL_list
do 
echo "sites|$PROJECT|$b@PERMANENT|$b|$META/$b.txt|||" >> $CGI_DIR/GL_list
echo "Not yet done. You should edit this file to enter relevant metadata" > "$META_DIR"/"$b".txt
done
cd ..




elif [ "$a" == windows ] 	# secondary condition 4
then 
cd $a
echo `pwd`

for b in `ls` 
do  
echo "region|$PROJECT|$b@PERMANENT|$b||||" >> $CGI_DIR/GL_list
done 
cd ..

fi	# End secondary conditions

done	# End secondary loop




cd ../.. # Back again in GRASS_DATA_DIR directory
echo `pwd`

let "COUNT += 1"  # Counter incremet








else	# NOT PERMANENT directory in supposed location

cd ..
echo `pwd`

fi	# End main condition 2

fi	# End main condition


done	# End main loop



# End making GL_list ####################










# Getting GL_list lines in order

grep raster $CGI_DIR/GL_list | sort >> $CGI_DIR/GL_list2
echo -e "\n\n\n\n" >> $CGI_DIR/GL_list2
grep vector $CGI_DIR/GL_list | sort >> $CGI_DIR/GL_list2
echo -e "\n\n\n\n" >> $CGI_DIR/GL_list2
grep sites $CGI_DIR/GL_list | sort >> $CGI_DIR/GL_list2
echo -e "\n\n\n\n" >> $CGI_DIR/GL_list2
grep region $CGI_DIR/GL_list | sort >> $CGI_DIR/GL_list2
echo -e "\n\n\n\n" >> $CGI_DIR/GL_list2
grep project $CGI_DIR/GL_list | sort >> $CGI_DIR/GL_list2

cp $CGI_DIR/GL_list2 $CGI_DIR/GL_list



# Make new GL_matrix.html

$DOCS_DIR/glinks/progs/glmatrix.pl $CGI_DIR/GL_list
cp GL_matrix.html $DOCS_DIR/glinks/GL_matrix.html 

# Finishing customized index.html

cat $DOCS_DIR/glinks/end.txt >> $DOCS_DIR/glinks/index.html

chown $NP_USER $DOCS_DIR/glinks/index.html

echo
echo
echo "Congratulations - install script complete."
echo "You may be ready to access http://localhost/glinks/index.html to test."
echo "Refer to glreadme.html if you have problems."
echo
echo


fi	# End of user's directories setting



############################################################

#  Work on user data done

############################################################










# Now the lines for no confirmation of user variable values

else 
echo
echo
echo
echo
echo "User aborted script. Installation has not been done"
echo
echo
echo
echo
fi
